# Generated by Django 5.2.6 on 2025-09-24 05:45

import django.utils.timezone
import django_revision.revision_field
import edc_identifier.managers
import edc_model.validators.date
import edc_protocol.validators
import edc_sites.managers
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_prn", "0066_alter_historicallosstofollowup_subject_identifier_and_more"),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="offschedule",
            managers=[
                ("on_site", edc_sites.managers.CurrentSiteManager()),
                ("objects", edc_identifier.managers.SubjectIdentifierManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="offscheduledmreferral",
            managers=[
                ("on_site", edc_sites.managers.CurrentSiteManager()),
                ("objects", edc_identifier.managers.SubjectIdentifierManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="offschedulepostnatal",
            managers=[
                ("on_site", edc_sites.managers.CurrentSiteManager()),
                ("objects", edc_identifier.managers.SubjectIdentifierManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name="offschedulepregnancy",
            managers=[
                ("on_site", edc_sites.managers.CurrentSiteManager()),
                ("objects", edc_identifier.managers.SubjectIdentifierManager()),
            ],
        ),
        migrations.AlterField(
            model_name="dmreferral",
            name="action_identifier",
            field=models.CharField(blank=True, default="", max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="dmreferral",
            name="action_item_reason",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="dmreferral",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="dmreferral",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="dmreferral",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="dmreferral",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="dmreferral",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="dmreferral",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="dmreferral",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="endofstudy",
            name="action_identifier",
            field=models.CharField(blank=True, default="", max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="endofstudy",
            name="action_item_reason",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="endofstudy",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="endofstudy",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="endofstudy",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="endofstudy",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="endofstudy",
            name="offstudy_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
                verbose_name="Off-study date and time",
            ),
        ),
        migrations.AlterField(
            model_name="endofstudy",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="endofstudy",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="endofstudy",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldmreferral",
            name="action_identifier",
            field=models.CharField(
                blank=True, db_index=True, default="", max_length=50
            ),
        ),
        migrations.AlterField(
            model_name="historicaldmreferral",
            name="action_item_reason",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="historicaldmreferral",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicaldmreferral",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicaldmreferral",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldmreferral",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldmreferral",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="historicaldmreferral",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="historicaldmreferral",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalendofstudy",
            name="action_identifier",
            field=models.CharField(
                blank=True, db_index=True, default="", max_length=50
            ),
        ),
        migrations.AlterField(
            model_name="historicalendofstudy",
            name="action_item_reason",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="historicalendofstudy",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalendofstudy",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalendofstudy",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalendofstudy",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalendofstudy",
            name="offstudy_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
                verbose_name="Off-study date and time",
            ),
        ),
        migrations.AlterField(
            model_name="historicalendofstudy",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="historicalendofstudy",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="historicalendofstudy",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicallosstofollowup",
            name="action_identifier",
            field=models.CharField(
                blank=True, db_index=True, default="", max_length=50
            ),
        ),
        migrations.AlterField(
            model_name="historicallosstofollowup",
            name="action_item_reason",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="historicallosstofollowup",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicallosstofollowup",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicallosstofollowup",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicallosstofollowup",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicallosstofollowup",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="historicallosstofollowup",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="historicallosstofollowup",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedule",
            name="action_identifier",
            field=models.CharField(
                blank=True, db_index=True, default="", max_length=50
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedule",
            name="action_item_reason",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="historicaloffschedule",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedule",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedule",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedule",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedule",
            name="offschedule_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
                verbose_name="Date and time subject taken off schedule",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedule",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedule",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedule",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffscheduledmreferral",
            name="action_identifier",
            field=models.CharField(
                blank=True, db_index=True, default="", max_length=50
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffscheduledmreferral",
            name="action_item_reason",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="historicaloffscheduledmreferral",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffscheduledmreferral",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffscheduledmreferral",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffscheduledmreferral",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffscheduledmreferral",
            name="offschedule_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
                verbose_name="Date and time subject taken off schedule",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffscheduledmreferral",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffscheduledmreferral",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffscheduledmreferral",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepostnatal",
            name="action_identifier",
            field=models.CharField(
                blank=True, db_index=True, default="", max_length=50
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepostnatal",
            name="action_item_reason",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepostnatal",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepostnatal",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepostnatal",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepostnatal",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepostnatal",
            name="offschedule_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
                verbose_name="Date and time subject taken off schedule",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepostnatal",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepostnatal",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepostnatal",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepregnancy",
            name="action_identifier",
            field=models.CharField(
                blank=True, db_index=True, default="", max_length=50
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepregnancy",
            name="action_item_reason",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepregnancy",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepregnancy",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepregnancy",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepregnancy",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepregnancy",
            name="offschedule_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
                verbose_name="Date and time subject taken off schedule",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepregnancy",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepregnancy",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffschedulepregnancy",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffstudymedication",
            name="action_identifier",
            field=models.CharField(
                blank=True, db_index=True, default="", max_length=50
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffstudymedication",
            name="action_item_reason",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="historicaloffstudymedication",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffstudymedication",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffstudymedication",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffstudymedication",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffstudymedication",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffstudymedication",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="historicaloffstudymedication",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedule",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedule",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedule",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedule",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedule",
            name="onschedule_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedule",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalonscheduledmreferral",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalonscheduledmreferral",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalonscheduledmreferral",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalonscheduledmreferral",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalonscheduledmreferral",
            name="onschedule_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
            ),
        ),
        migrations.AlterField(
            model_name="historicalonscheduledmreferral",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedulepostnatal",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedulepostnatal",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedulepostnatal",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedulepostnatal",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedulepostnatal",
            name="onschedule_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedulepostnatal",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedulepregnancy",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedulepregnancy",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedulepregnancy",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedulepregnancy",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedulepregnancy",
            name="onschedule_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
            ),
        ),
        migrations.AlterField(
            model_name="historicalonschedulepregnancy",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalpregnancynotification",
            name="action_identifier",
            field=models.CharField(
                blank=True, db_index=True, default="", max_length=50
            ),
        ),
        migrations.AlterField(
            model_name="historicalpregnancynotification",
            name="action_item_reason",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="historicalpregnancynotification",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalpregnancynotification",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalpregnancynotification",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalpregnancynotification",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalpregnancynotification",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="historicalpregnancynotification",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="historicalpregnancynotification",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="action_identifier",
            field=models.CharField(
                blank=True, db_index=True, default="", max_length=50
            ),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="action_item_reason",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="corrective_action",
            field=models.TextField(
                blank=True, default="", verbose_name="Corrective action taken"
            ),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="incident_description",
            field=models.TextField(
                default="",
                help_text="Describe in full. Explain how the incident happened, what occurred, etc.",
                verbose_name="Describe the incident",
            ),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="incident_other",
            field=models.CharField(
                blank=True,
                default="",
                max_length=75,
                verbose_name="If other, please specify",
            ),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="incident_reason",
            field=models.TextField(
                default="", verbose_name="Explain the reason why the incident occurred"
            ),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="preventative_action",
            field=models.TextField(
                blank=True, default="", verbose_name="Preventative action taken"
            ),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="reasons_withdrawn",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="report_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now, verbose_name="Report Date and Time"
            ),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="safety_impact_details",
            field=models.TextField(
                blank=True, default="", verbose_name='If "Yes", provide details'
            ),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="short_description",
            field=models.CharField(
                default="",
                help_text="Max 35 characters. Note: there is additional space below for a more detailed description",
                max_length=35,
                verbose_name="Provide a short description of this incident",
            ),
        ),
        migrations.AlterField(
            model_name="historicalprotocolincident",
            name="study_outcomes_impact_details",
            field=models.TextField(
                blank=True, default="", verbose_name='If "Yes", provide details'
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjecttransfer",
            name="action_identifier",
            field=models.CharField(
                blank=True, db_index=True, default="", max_length=50
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjecttransfer",
            name="action_item_reason",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="historicalsubjecttransfer",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjecttransfer",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjecttransfer",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjecttransfer",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjecttransfer",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjecttransfer",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjecttransfer",
            name="report_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now, verbose_name="Report Date and Time"
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjecttransfer",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjecttransfer",
            name="transfer_date",
            field=models.DateField(
                default=django.utils.timezone.now, verbose_name="Transfer date"
            ),
        ),
        migrations.AlterField(
            model_name="losstofollowup",
            name="action_identifier",
            field=models.CharField(blank=True, default="", max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="losstofollowup",
            name="action_item_reason",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="losstofollowup",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="losstofollowup",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="losstofollowup",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="losstofollowup",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="losstofollowup",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="losstofollowup",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="losstofollowup",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="offschedule",
            name="action_identifier",
            field=models.CharField(blank=True, default="", max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="offschedule",
            name="action_item_reason",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="offschedule",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="offschedule",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="offschedule",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="offschedule",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="offschedule",
            name="offschedule_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
                verbose_name="Date and time subject taken off schedule",
            ),
        ),
        migrations.AlterField(
            model_name="offschedule",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="offschedule",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="offschedule",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="offscheduledmreferral",
            name="action_identifier",
            field=models.CharField(blank=True, default="", max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="offscheduledmreferral",
            name="action_item_reason",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="offscheduledmreferral",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="offscheduledmreferral",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="offscheduledmreferral",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="offscheduledmreferral",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="offscheduledmreferral",
            name="offschedule_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
                verbose_name="Date and time subject taken off schedule",
            ),
        ),
        migrations.AlterField(
            model_name="offscheduledmreferral",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="offscheduledmreferral",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="offscheduledmreferral",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="offschedulepostnatal",
            name="action_identifier",
            field=models.CharField(blank=True, default="", max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="offschedulepostnatal",
            name="action_item_reason",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="offschedulepostnatal",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="offschedulepostnatal",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="offschedulepostnatal",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="offschedulepostnatal",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="offschedulepostnatal",
            name="offschedule_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
                verbose_name="Date and time subject taken off schedule",
            ),
        ),
        migrations.AlterField(
            model_name="offschedulepostnatal",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="offschedulepostnatal",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="offschedulepostnatal",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="offschedulepregnancy",
            name="action_identifier",
            field=models.CharField(blank=True, default="", max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="offschedulepregnancy",
            name="action_item_reason",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="offschedulepregnancy",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="offschedulepregnancy",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="offschedulepregnancy",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="offschedulepregnancy",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="offschedulepregnancy",
            name="offschedule_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
                verbose_name="Date and time subject taken off schedule",
            ),
        ),
        migrations.AlterField(
            model_name="offschedulepregnancy",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="offschedulepregnancy",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="offschedulepregnancy",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="offstudymedication",
            name="action_identifier",
            field=models.CharField(blank=True, default="", max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="offstudymedication",
            name="action_item_reason",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="offstudymedication",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="offstudymedication",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="offstudymedication",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="offstudymedication",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="offstudymedication",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="offstudymedication",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="offstudymedication",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="onschedule",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="onschedule",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="onschedule",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="onschedule",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="onschedule",
            name="onschedule_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
            ),
        ),
        migrations.AlterField(
            model_name="onschedule",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="onscheduledmreferral",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="onscheduledmreferral",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="onscheduledmreferral",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="onscheduledmreferral",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="onscheduledmreferral",
            name="onschedule_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
            ),
        ),
        migrations.AlterField(
            model_name="onscheduledmreferral",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="onschedulepostnatal",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="onschedulepostnatal",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="onschedulepostnatal",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="onschedulepostnatal",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="onschedulepostnatal",
            name="onschedule_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
            ),
        ),
        migrations.AlterField(
            model_name="onschedulepostnatal",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="onschedulepregnancy",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="onschedulepregnancy",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="onschedulepregnancy",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="onschedulepregnancy",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="onschedulepregnancy",
            name="onschedule_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                validators=[
                    edc_protocol.validators.datetime_not_before_study_start,
                    edc_model.validators.date.datetime_not_future,
                ],
            ),
        ),
        migrations.AlterField(
            model_name="onschedulepregnancy",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="pregnancynotification",
            name="action_identifier",
            field=models.CharField(blank=True, default="", max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="pregnancynotification",
            name="action_item_reason",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="pregnancynotification",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="pregnancynotification",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="pregnancynotification",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="pregnancynotification",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="pregnancynotification",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="pregnancynotification",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="pregnancynotification",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="action_identifier",
            field=models.CharField(blank=True, default="", max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="action_item_reason",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="corrective_action",
            field=models.TextField(
                blank=True, default="", verbose_name="Corrective action taken"
            ),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="incident_description",
            field=models.TextField(
                default="",
                help_text="Describe in full. Explain how the incident happened, what occurred, etc.",
                verbose_name="Describe the incident",
            ),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="incident_other",
            field=models.CharField(
                blank=True,
                default="",
                max_length=75,
                verbose_name="If other, please specify",
            ),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="incident_reason",
            field=models.TextField(
                default="", verbose_name="Explain the reason why the incident occurred"
            ),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="preventative_action",
            field=models.TextField(
                blank=True, default="", verbose_name="Preventative action taken"
            ),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="reasons_withdrawn",
            field=models.TextField(blank=True, default=""),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="report_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now, verbose_name="Report Date and Time"
            ),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="safety_impact_details",
            field=models.TextField(
                blank=True, default="", verbose_name='If "Yes", provide details'
            ),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="short_description",
            field=models.CharField(
                default="",
                help_text="Max 35 characters. Note: there is additional space below for a more detailed description",
                max_length=35,
                verbose_name="Provide a short description of this incident",
            ),
        ),
        migrations.AlterField(
            model_name="protocolincident",
            name="study_outcomes_impact_details",
            field=models.TextField(
                blank=True, default="", verbose_name='If "Yes", provide details'
            ),
        ),
        migrations.AlterField(
            model_name="subjecttransfer",
            name="action_identifier",
            field=models.CharField(blank=True, default="", max_length=50, unique=True),
        ),
        migrations.AlterField(
            model_name="subjecttransfer",
            name="action_item_reason",
            field=models.TextField(default="", editable=False),
        ),
        migrations.AlterField(
            model_name="subjecttransfer",
            name="device_created",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device created"
            ),
        ),
        migrations.AlterField(
            model_name="subjecttransfer",
            name="device_modified",
            field=models.CharField(
                blank=True, default="", max_length=10, verbose_name="Device modified"
            ),
        ),
        migrations.AlterField(
            model_name="subjecttransfer",
            name="locale_created",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale created",
            ),
        ),
        migrations.AlterField(
            model_name="subjecttransfer",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AlterField(
            model_name="subjecttransfer",
            name="parent_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to parent reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="subjecttransfer",
            name="related_action_identifier",
            field=models.CharField(
                blank=True,
                default="",
                help_text="action identifier that links to related reference model instance.",
                max_length=30,
            ),
        ),
        migrations.AlterField(
            model_name="subjecttransfer",
            name="report_datetime",
            field=models.DateTimeField(
                default=django.utils.timezone.now, verbose_name="Report Date and Time"
            ),
        ),
        migrations.AlterField(
            model_name="subjecttransfer",
            name="revision",
            field=django_revision.revision_field.RevisionField(
                blank=True,
                default="",
                editable=False,
                help_text="System field. From git repository (tag:branch:commit), project metadata, project toml, project VERSION, or settings.",
                max_length=75,
                verbose_name="Revision",
            ),
        ),
        migrations.AlterField(
            model_name="subjecttransfer",
            name="transfer_date",
            field=models.DateField(
                default=django.utils.timezone.now, verbose_name="Transfer date"
            ),
        ),
    ]



# Generated by Django 4.0.5 on 2022-06-27 14:17

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("meta_prn", "0030_auto_20220627_1119"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="historicaloffschedule",
            options={
                "get_latest_by": "history_date",
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Off-schedule",
            },
        ),
        migrations.AlterModelOptions(
            name="historicaloffschedulepostnatal",
            options={
                "get_latest_by": "history_date",
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Off-schedule: post-natal",
            },
        ),
        migrations.AlterModelOptions(
            name="historicaloffschedulepregnancy",
            options={
                "get_latest_by": "history_date",
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Off-schedule: Pregnancy",
            },
        ),
        migrations.AlterModelOptions(
            name="offschedule",
            options={"verbose_name": "Off-schedule", "verbose_name_plural": "Off-schedule"},
        ),
        migrations.AlterModelOptions(
            name="offschedulepostnatal",
            options={
                "verbose_name": "Off-schedule: post-natal",
                "verbose_name_plural": "Off-schedule: post-natal",
            },
        ),
        migrations.AlterModelOptions(
            name="offschedulepregnancy",
            options={
                "verbose_name": "Off-schedule: Pregnancy",
                "verbose_name_plural": "Off-schedule: Pregnancy",
            },
        ),
    ]



import sys

from django.core.exceptions import ObjectDoesNotExist
from django.core.management import color_style

from meta_pharmacy.constants import METFORMIN

style = color_style()


def prepare_meta_pharmacy():
    from django.apps import apps as django_apps  # noqa: PLC0415

    sys.stdout.write(style.MIGRATE_HEADING("Populating static pharmacy models for META3:\n"))

    medication_model_cls = django_apps.get_model("edc_pharmacy.medication")
    formulation_model_cls = django_apps.get_model("edc_pharmacy.formulation")
    dosage_guideline_model_cls = django_apps.get_model("edc_pharmacy.dosageguideline")
    route_model_cls = django_apps.get_model("edc_pharmacy.route")
    units_model_cls = django_apps.get_model("edc_pharmacy.units")
    formulation_type_model_cls = django_apps.get_model("edc_pharmacy.formulationtype")
    frequency_units_model_cls = django_apps.get_model("edc_pharmacy.frequencyunits")

    units = units_model_cls.objects.get(name="mg")
    formulation_type = formulation_type_model_cls.objects.get(name="tablet")
    route = route_model_cls.objects.get(name="oral")
    frequency_units = frequency_units_model_cls.objects.get(name="day")

    # medication

    try:
        medication_obj = medication_model_cls.objects.get(name=METFORMIN)
    except ObjectDoesNotExist:
        medication_obj = medication_model_cls.objects.create(
            name=METFORMIN, display_name="Metformin"
        )
    else:
        medication_obj.display_name = "Metformin"
        medication_obj.save()

    # formulation
    try:
        formulation_obj = formulation_model_cls.objects.get(
            medication=medication_obj, strength=500, units=units
        )
    except ObjectDoesNotExist:
        formulation_model_cls.objects.create(
            medication=medication_obj,
            strength=500,
            units=units,
            formulation_type=formulation_type,
            route=route,
        )
    else:
        formulation_obj.strength = 500
        formulation_obj.units = units
        formulation_obj.formulation_type = formulation_type
        formulation_obj.route = route
        formulation_obj.save()

    # dosage guideline
    for dose in [1000, 2000]:
        try:
            dosage_guideline = dosage_guideline_model_cls.objects.get(
                medication=medication_obj, dose=dose, dose_units=units
            )
        except ObjectDoesNotExist:
            dosage_guideline_model_cls.objects.create(
                medication=medication_obj,
                dose=dose,
                dose_units=units,
                frequency=1,
                frequency_units=frequency_units,
            )
        else:
            dosage_guideline.medication = medication_obj
            dosage_guideline.dose = dose
            dosage_guideline.dose_units = units
            dosage_guideline.frequency = 1
            dosage_guideline.frequency_units = frequency_units
            dosage_guideline.save()



# Generated by Django 3.0.6 on 2020-05-16 22:25

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0033_auto_20200516_2356"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="historicalmissedvisit",
            options={
                "get_latest_by": "history_date",
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Missed Visit Report",
            },
        ),
        migrations.AlterModelOptions(
            name="missedvisit",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Missed Visit Report",
                "verbose_name_plural": "Missed Visit Report",
            },
        ),
    ]



from .dm_referral import DmReferral
from .end_of_study import EndOfStudy
from .loss_to_followup import LossToFollowup
from .off_study_medication import OffStudyMedication
from .offschedule import (
    OffSchedule,
    OffScheduleDmReferral,
    OffSchedulePostnatal,
    OffSchedulePregnancy,
)
from .onschedule import (
    OnSchedule,
    OnScheduleDmReferral,
    OnSchedulePostnatal,
    OnSchedulePregnancy,
)
from .pregnancy_notification import PregnancyNotification
from .protocol_incident import ProtocolIncident
from .signals import (
    update_schedule_on_pregnancy_notification_post_save,
    update_urine_pregnancy_on_pregnancy_notification_on_post_save,
)
from .subject_transfer import SubjectTransfer

__all__ = [
    "DmReferral",
    "EndOfStudy",
    "LossToFollowup",
    "OffSchedule",
    "OffScheduleDmReferral",
    "OffSchedulePostnatal",
    "OffSchedulePregnancy",
    "OffStudyMedication",
    "OnSchedule",
    "OnScheduleDmReferral",
    "OnSchedulePostnatal",
    "OnSchedulePregnancy",
    "PregnancyNotification",
    "ProtocolIncident",
    "SubjectTransfer",
    "update_schedule_on_pregnancy_notification_post_save",
    "update_urine_pregnancy_on_pregnancy_notification_on_post_save",
]



# Generated by Django 4.2.10 on 2024-03-08 02:24

import _socket
from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django.utils.timezone
import django_crypto_fields.fields.encrypted_char_field
import django_crypto_fields.fields.encrypted_text_field
import django_crypto_fields.fields.firstname_field
import django_crypto_fields.fields.identity_field
import django_crypto_fields.fields.lastname_field
import django_revision.revision_field
import edc_consent.validators
import edc_model.validators.date
import edc_model_fields.fields.date_estimated
import edc_protocol.validators
import simple_history.models
import uuid


class Migration(migrations.Migration):
    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("meta_consent", "0023_subjectconsentv1_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="HistoricalSubjectConsentV1",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "modified",
                    models.DateTimeField(blank=True, default=django.utils.timezone.now),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                        verbose_name="Hostname created",
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                        verbose_name="Hostname modified",
                    ),
                ),
                (
                    "device_created",
                    models.CharField(blank=True, max_length=10, verbose_name="Device created"),
                ),
                (
                    "device_modified",
                    models.CharField(
                        blank=True, max_length=10, verbose_name="Device modified"
                    ),
                ),
                (
                    "locale_created",
                    models.CharField(
                        blank=True,
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        null=True,
                        verbose_name="Locale created",
                    ),
                ),
                (
                    "locale_modified",
                    models.CharField(
                        blank=True,
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        null=True,
                        verbose_name="Locale modified",
                    ),
                ),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50)),
                (
                    "subject_identifier_as_pk",
                    models.UUIDField(default=uuid.uuid4, editable=False),
                ),
                (
                    "subject_identifier_aka",
                    models.CharField(
                        editable=False,
                        help_text="track a previously allocated identifier.",
                        max_length=50,
                        null=True,
                        verbose_name="Subject Identifier a.k.a",
                    ),
                ),
                (
                    "citizen",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=3,
                        verbose_name="Is the participant a Botswana citizen? ",
                    ),
                ),
                (
                    "legal_marriage",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        help_text="If 'No', participant may not be consented.",
                        max_length=3,
                        null=True,
                        verbose_name="If not a citizen, is the participant legally married to a Botswana citizen?",
                    ),
                ),
                (
                    "marriage_certificate",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                        default="N/A",
                        help_text="If 'No', participant may not be consented.",
                        max_length=3,
                        null=True,
                        verbose_name="[Interviewer] Has the participant produced the marriage certificate as proof? ",
                    ),
                ),
                (
                    "marriage_certificate_no",
                    models.CharField(
                        blank=True,
                        help_text="e.g. 000/YYYY",
                        max_length=9,
                        null=True,
                        verbose_name="What is the marriage certificate number?",
                    ),
                ),
                (
                    "identity",
                    django_crypto_fields.fields.identity_field.IdentityField(
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        verbose_name="Identity number",
                    ),
                ),
                (
                    "identity_type",
                    models.CharField(
                        choices=[
                            ("country_id", "Country ID number"),
                            ("drivers", "Driver's license"),
                            ("passport", "Passport"),
                            ("hospital_no", "Hospital number"),
                            ("country_id_rcpt", "Country ID receipt"),
                            ("mobile_no", "Mobile number"),
                            ("OTHER", "Other"),
                        ],
                        max_length=25,
                        verbose_name="What type of identity number is this?",
                    ),
                ),
                (
                    "confirm_identity",
                    django_crypto_fields.fields.identity_field.IdentityField(
                        help_text="Retype the identity number (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                    ),
                ),
                ("dob", models.DateField(null=True, verbose_name="Date of birth")),
                (
                    "is_dob_estimated",
                    edc_model_fields.fields.date_estimated.IsDateEstimatedField(
                        choices=[
                            ("-", "No"),
                            ("D", "Yes, estimated the Day"),
                            ("MD", "Yes, estimated Month and Day"),
                            ("YMD", "Yes, estimated Year, Month and Day"),
                        ],
                        help_text="If the exact date is not known, please indicate which part of the date is estimated.",
                        max_length=25,
                        null=True,
                        verbose_name="Is date of birth estimated?",
                    ),
                ),
                (
                    "gender",
                    models.CharField(
                        choices=[("M", "Male"), ("F", "Female"), ("U", "Undetermined")],
                        max_length=1,
                        null=True,
                        verbose_name="Gender",
                    ),
                ),
                (
                    "guardian_name",
                    django_crypto_fields.fields.lastname_field.LastnameField(
                        blank=True,
                        help_text="Required only if participant is a minor.<BR>Format is 'LASTNAME, FIRSTNAME'. All uppercase separated by a comma. (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        validators=[edc_consent.validators.FullNameValidator()],
                        verbose_name="Guardian's last and first name",
                    ),
                ),
                ("subject_type", models.CharField(max_length=25)),
                (
                    "first_name",
                    django_crypto_fields.fields.firstname_field.FirstnameField(
                        blank=True,
                        help_text="Use UPPERCASE letters only. (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        validators=[
                            django.core.validators.RegexValidator(
                                message="Ensure name consist of letters only in upper case",
                                regex="^([A-Z]+$|[A-Z]+\\ [A-Z]+)$",
                            )
                        ],
                    ),
                ),
                (
                    "last_name",
                    django_crypto_fields.fields.lastname_field.LastnameField(
                        blank=True,
                        help_text="Use UPPERCASE letters only. (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        validators=[
                            django.core.validators.RegexValidator(
                                message="Ensure name consist of letters only in upper case",
                                regex="^([A-Z]+$|[A-Z]+\\ [A-Z]+)$",
                            )
                        ],
                        verbose_name="Surname",
                    ),
                ),
                (
                    "initials",
                    django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                        blank=True,
                        help_text=" (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        validators=[
                            django.core.validators.RegexValidator(
                                message="Ensure initials consist of letters only in upper case, no spaces.",
                                regex="^[A-Z]{2,3}$",
                            )
                        ],
                    ),
                ),
                (
                    "consent_reviewed",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        help_text="If no, participant is not eligible.",
                        max_length=3,
                        null=True,
                        validators=[edc_consent.validators.eligible_if_yes],
                        verbose_name="I have reviewed the consent with the participant",
                    ),
                ),
                (
                    "study_questions",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        help_text="If no, participant is not eligible.",
                        max_length=3,
                        null=True,
                        validators=[edc_consent.validators.eligible_if_yes],
                        verbose_name="I have answered all questions the participant had about the study",
                    ),
                ),
                (
                    "assessment_score",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        help_text="If no, participant is not eligible.",
                        max_length=3,
                        null=True,
                        validators=[edc_consent.validators.eligible_if_yes],
                        verbose_name="I have asked the participant questions about this study and the participant has demonstrated understanding",
                    ),
                ),
                (
                    "consent_signature",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        help_text="If no, participant is not eligible.",
                        max_length=3,
                        null=True,
                        validators=[edc_consent.validators.eligible_if_yes],
                        verbose_name="I have verified that the participant has signed the consent form",
                    ),
                ),
                (
                    "consent_copy",
                    models.CharField(
                        choices=[
                            ("Yes", "Yes"),
                            ("No", "No"),
                            ("Declined", "Yes, but subject declined copy"),
                        ],
                        help_text="If declined, return copy with the consent",
                        max_length=20,
                        null=True,
                        validators=[edc_consent.validators.eligible_if_yes_or_declined],
                        verbose_name="I have provided the participant with a copy of their signed informed consent",
                    ),
                ),
                (
                    "may_store_genetic_samples",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=25,
                        verbose_name="Does the participant agree that a portion of the blood sample that is taken be stored for genetic analysis?",
                    ),
                ),
                (
                    "may_store_samples",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        max_length=3,
                        verbose_name="Does the participant agree to have samples stored after the study has ended",
                    ),
                ),
                ("is_verified", models.BooleanField(default=False, editable=False)),
                ("is_verified_datetime", models.DateTimeField(editable=False, null=True)),
                ("verified_by", models.CharField(editable=False, max_length=25, null=True)),
                (
                    "is_incarcerated",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        help_text="( if 'Yes' STOP participant cannot be consented )",
                        max_length=3,
                        null=True,
                        validators=[edc_consent.validators.eligible_if_no],
                        verbose_name="Is the participant under involuntary incarceration?",
                    ),
                ),
                (
                    "is_literate",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        help_text="If 'No' provide witness's name on this form and signature on the paper document.",
                        max_length=3,
                        verbose_name="Is the participant literate?",
                    ),
                ),
                (
                    "witness_name",
                    django_crypto_fields.fields.lastname_field.LastnameField(
                        blank=True,
                        help_text="Required only if participant is illiterate.<br>Format is 'LASTNAME, FIRSTNAME'. All uppercase separated by a comma. (Encryption: RSA local)",
                        max_length=71,
                        null=True,
                        validators=[edc_consent.validators.FullNameValidator()],
                        verbose_name="Witness's last and first name",
                    ),
                ),
                (
                    "language",
                    models.CharField(
                        choices=[
                            ("sw", "Swahili"),
                            ("en-gb", "British English"),
                            ("en", "English"),
                            ("mas", "Maasai"),
                        ],
                        help_text="The language used for the consent process will also be used during data collection.",
                        max_length=25,
                        verbose_name="Language of consent",
                    ),
                ),
                (
                    "slug",
                    models.CharField(
                        db_index=True,
                        default="",
                        editable=False,
                        help_text="a field used for quick search",
                        max_length=250,
                        null=True,
                    ),
                ),
                (
                    "version",
                    models.CharField(
                        editable=False,
                        help_text="See 'consent definition' for consent versions by period.",
                        max_length=10,
                        verbose_name="Consent version",
                    ),
                ),
                ("update_versions", models.BooleanField(default=False)),
                (
                    "consent_definition_name",
                    models.CharField(
                        editable=False,
                        max_length=50,
                        null=True,
                        verbose_name="Consent definition",
                    ),
                ),
                (
                    "model_name",
                    models.CharField(
                        editable=False,
                        help_text="label_lower of this model class. Will be different if instance has been added/edited via a proxy model",
                        max_length=50,
                        null=True,
                        verbose_name="model",
                    ),
                ),
                (
                    "consent_datetime",
                    models.DateTimeField(
                        validators=[
                            edc_protocol.validators.datetime_not_before_study_start,
                            edc_model.validators.date.datetime_not_future,
                        ],
                        verbose_name="Consent date and time",
                    ),
                ),
                ("report_datetime", models.DateTimeField(editable=False, null=True)),
                (
                    "sid",
                    models.CharField(
                        blank=True,
                        editable=False,
                        help_text="Used for randomization against a prepared rando-list.",
                        max_length=15,
                        null=True,
                        verbose_name="SID",
                    ),
                ),
                (
                    "comment",
                    django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                        blank=True,
                        help_text=" (Encryption: AES local)",
                        max_length=250,
                        null=True,
                        verbose_name="Comment",
                    ),
                ),
                (
                    "dm_comment",
                    models.CharField(
                        editable=False,
                        help_text="see also edc.data manager.",
                        max_length=150,
                        null=True,
                        verbose_name="Data Management comment",
                    ),
                ),
                (
                    "consent_identifier",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        help_text="A unique identifier for this consent instance",
                    ),
                ),
                (
                    "screening_identifier",
                    models.CharField(
                        db_index=True, max_length=50, verbose_name="Screening identifier"
                    ),
                ),
                (
                    "screening_datetime",
                    models.DateTimeField(
                        editable=False, null=True, verbose_name="Screening datetime"
                    ),
                ),
                (
                    "ethnicity",
                    models.CharField(
                        editable=False, help_text="from screening", max_length=15, null=True
                    ),
                ),
                (
                    "completed_by_next_of_kin",
                    models.CharField(
                        choices=[("Yes", "Yes"), ("No", "No")],
                        default="No",
                        editable=False,
                        max_length=10,
                    ),
                ),
                (
                    "study_medication_name",
                    models.CharField(
                        default="metformin",
                        editable=False,
                        help_text="Used by `create_prescription` in the signal",
                        max_length=25,
                    ),
                ),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4, editable=False, primary_key=True, serialize=False
                    ),
                ),
                ("history_date", models.DateTimeField(db_index=True)),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical Consent V1",
                "verbose_name_plural": "historical Consent V1",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": ("history_date", "history_id"),
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
    ]



from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from django_crypto_fields.fields import EncryptedCharField
from edc_constants.choices import GENDER, YES_NO, YES_NO_NA
from edc_model.models import BaseUuidModel, HistoricalRecords
from edc_sites.managers import CurrentSiteManager
from edc_sites.model_mixins import SiteModelMixin
from edc_utils.date import get_utcnow

from ..choices import ETHNICITY
from .subject_screening import SubjectScreening


class IcpReferralError(Exception):
    pass


class IcpReferralManager(models.Manager):
    def get_by_natural_key(self, screening_identifier):
        return self.get(screening_identifier=screening_identifier)


class IcpReferral(SiteModelMixin, BaseUuidModel):
    """Not used"""

    subject_screening = models.OneToOneField(
        SubjectScreening, null=True, on_delete=models.PROTECT
    )

    screening_identifier = models.CharField(
        verbose_name="META Screening Identifier", max_length=25, unique=True
    )

    report_datetime = models.DateTimeField(
        verbose_name="Report Date and Time",
        default=get_utcnow,
        help_text="Date and time of report.",
    )

    hospital_identifier = EncryptedCharField(null=True, blank=True)

    gender = models.CharField(choices=GENDER, max_length=10)

    age_in_years = models.IntegerField()

    initials = models.CharField(max_length=3)

    ethnicity = models.CharField(
        max_length=15, choices=ETHNICITY, help_text="Used for eGFR calculation"
    )

    hiv_pos = models.CharField(verbose_name="HIV positive", max_length=15, choices=YES_NO)

    art_six_months = models.CharField(
        verbose_name="On anti-retroviral therapy for at least 6 months",
        max_length=15,
        choices=YES_NO_NA,
    )

    fbg_value = models.DecimalField(
        verbose_name="FBG level",
        max_digits=8,
        decimal_places=4,
        null=True,
        help_text="mmol/L",
    )

    hba1c_value = models.DecimalField(
        verbose_name="HbA1c",
        max_digits=8,
        decimal_places=4,
        null=True,
        help_text="in %",
    )

    ogtt_value = models.DecimalField(
        verbose_name="Blood glucose levels 2-hours after glucose solution given",
        max_digits=8,
        decimal_places=4,
        null=True,
        help_text="mmol/L",
    )

    meta_eligible = models.BooleanField(verbose_name="META eligibile")

    meta_eligibility_datetime = models.DateTimeField(
        null=True, help_text="Date and time META eligibility was determined"
    )

    referred = models.BooleanField(verbose_name="Referred", default=False)

    referred_datetime = models.DateTimeField(null=True, help_text="Date and time of referral")

    referral_reasons = models.TextField(default="")

    objects = IcpReferralManager()

    on_site = CurrentSiteManager()

    history = HistoricalRecords(inherit=True)

    def save(self, *args, **kwargs):
        try:
            SubjectScreening.objects.get(screening_identifier=self.screening_identifier)
        except ObjectDoesNotExist as e:
            raise IcpReferralError(
                f"Invalid META screening identifier. Got {self.screening_identifier}"
            ) from e
        super().save(*args, **kwargs)

    def __str__(self):
        return f"{self.screening_identifier} {self.gender} {self.age_in_years}"

    def natural_key(self):
        return (self.screening_identifier,)

    class Meta:
        pass



# Generated by Django 6.0 on 2025-04-15 22:06

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0221_auto_20250402_1913"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalstudymedication",
            name="stock_codes",
            field=models.TextField(
                blank=True,
                help_text="Enter the medication bottle barcode or barcodes. Type one code per line",
                max_length=30,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        message="Enter one or more valid codes, one code per line",
                        regex="^([A-Z0-9]{6})(\r\n[A-Z0-9]{6})*$",
                    )
                ],
            ),
        ),
        migrations.AlterField(
            model_name="studymedication",
            name="stock_codes",
            field=models.TextField(
                blank=True,
                help_text="Enter the medication bottle barcode or barcodes. Type one code per line",
                max_length=30,
                null=True,
                validators=[
                    django.core.validators.RegexValidator(
                        message="Enter one or more valid codes, one code per line",
                        regex="^([A-Z0-9]{6})(\r\n[A-Z0-9]{6})*$",
                    )
                ],
            ),
        ),
    ]



# Generated by Django 5.1.2 on 2025-01-15 23:45

import edc_consent.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_consent", "0027_auto_20250111_0344"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalsubjectconsentv1ext",
            name="assessment_score",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="If no, participant is not eligible.",
                max_length=3,
                null=True,
                validators=[edc_consent.validators.eligible_if_yes],
                verbose_name="I have asked the participant questions about this study and the participant has demonstrated understanding",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectconsentv1ext",
            name="consent_copy",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("Declined", "Yes, but subject declined copy"),
                ],
                help_text="If declined, return copy with the consent",
                max_length=20,
                null=True,
                validators=[edc_consent.validators.eligible_if_yes_or_declined],
                verbose_name="I have provided the participant with a copy of their signed informed consent",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectconsentv1ext",
            name="consent_reviewed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="If no, participant is not eligible.",
                max_length=3,
                null=True,
                validators=[edc_consent.validators.eligible_if_yes],
                verbose_name="I have reviewed the consent with the participant",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectconsentv1ext",
            name="consent_signature",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="If no, participant is not eligible.",
                max_length=3,
                null=True,
                validators=[edc_consent.validators.eligible_if_yes],
                verbose_name="I have verified that the participant has signed the consent form",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectconsentv1ext",
            name="study_questions",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="If no, participant is not eligible.",
                max_length=3,
                null=True,
                validators=[edc_consent.validators.eligible_if_yes],
                verbose_name="I have answered all questions the participant had about the study",
            ),
        ),
        migrations.AddField(
            model_name="subjectconsentv1ext",
            name="assessment_score",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="If no, participant is not eligible.",
                max_length=3,
                null=True,
                validators=[edc_consent.validators.eligible_if_yes],
                verbose_name="I have asked the participant questions about this study and the participant has demonstrated understanding",
            ),
        ),
        migrations.AddField(
            model_name="subjectconsentv1ext",
            name="consent_copy",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("Declined", "Yes, but subject declined copy"),
                ],
                help_text="If declined, return copy with the consent",
                max_length=20,
                null=True,
                validators=[edc_consent.validators.eligible_if_yes_or_declined],
                verbose_name="I have provided the participant with a copy of their signed informed consent",
            ),
        ),
        migrations.AddField(
            model_name="subjectconsentv1ext",
            name="consent_reviewed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="If no, participant is not eligible.",
                max_length=3,
                null=True,
                validators=[edc_consent.validators.eligible_if_yes],
                verbose_name="I have reviewed the consent with the participant",
            ),
        ),
        migrations.AddField(
            model_name="subjectconsentv1ext",
            name="consent_signature",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="If no, participant is not eligible.",
                max_length=3,
                null=True,
                validators=[edc_consent.validators.eligible_if_yes],
                verbose_name="I have verified that the participant has signed the consent form",
            ),
        ),
        migrations.AddField(
            model_name="subjectconsentv1ext",
            name="study_questions",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                help_text="If no, participant is not eligible.",
                max_length=3,
                null=True,
                validators=[edc_consent.validators.eligible_if_yes],
                verbose_name="I have answered all questions the participant had about the study",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectconsentv1ext",
            name="agrees_to_extension",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                help_text="See above for the definition of extended followup.",
                max_length=15,
                verbose_name="Does the participant agree to extend followup as per the protocol amendment?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectconsentv1ext",
            name="agrees_to_extension",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                help_text="See above for the definition of extended followup.",
                max_length=15,
                verbose_name="Does the participant agree to extend followup as per the protocol amendment?",
            ),
        ),
    ]



# Generated by Django 5.2.10 on 2026-01-23 14:00

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_lists", "0021_diagnosticdevices"),
        ("meta_subject", "0241_remove_glucose_diagnostic_device_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="glucose",
            name="ogtt_diagnostic_device",
            field=models.ForeignKey(
                limit_choices_to={"name__in": ["accuchek", "hemocue", "OTHER", "N/A"]},
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="ogtt_diagnostic_device",
                to="meta_lists.diagnosticdevices",
            ),
        ),
        migrations.AddField(
            model_name="historicalglucose",
            name="ogtt_diagnostic_device",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"name__in": ["accuchek", "hemocue", "OTHER", "N/A"]},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="meta_lists.diagnosticdevices",
            ),
        ),
        migrations.AlterField(
            model_name="glucose",
            name="fbg_diagnostic_device",
            field=models.ForeignKey(
                limit_choices_to={"name__in": ["accuchek", "hemocue", "OTHER", "N/A"]},
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="fbg_diagnostic_device",
                to="meta_lists.diagnosticdevices",
            ),
        ),
        migrations.AlterField(
            model_name="glucosefbg",
            name="fbg_diagnostic_device",
            field=models.ForeignKey(
                limit_choices_to={"name__in": ["accuchek", "hemocue", "OTHER", "N/A"]},
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="meta_lists.diagnosticdevices",
            ),
        ),
        migrations.AlterField(
            model_name="historicalglucose",
            name="fbg_diagnostic_device",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"name__in": ["accuchek", "hemocue", "OTHER", "N/A"]},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="meta_lists.diagnosticdevices",
            ),
        ),
        migrations.AlterField(
            model_name="historicalglucosefbg",
            name="fbg_diagnostic_device",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"name__in": ["accuchek", "hemocue", "OTHER", "N/A"]},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="meta_lists.diagnosticdevices",
            ),
        ),
    ]



import pandas as pd
from great_tables import GT, html, loc, style


def df_as_great_table(
    df_table: pd.DataFrame,
    title=None,
    subtitle=None,
):
    """Used by monitoring report"""
    return (
        GT(df_table)
        .tab_header(title=html(f'<div class="table-header">{title}</div>'), subtitle=subtitle)
        .cols_align(align="left", columns=[0, 1])
        .cols_align(align="right", columns=list(range(2, len(df_table.columns))))
        .opt_stylize(style=3)
        .opt_row_striping(row_striping=True)
        .opt_vertical_padding(scale=1.2)
        .opt_horizontal_padding(scale=1.0)
        .tab_options(
            table_width="100%",
            stub_background_color="snow",
            row_group_border_bottom_style="hidden",
            row_group_padding=0.5,
            row_group_background_color="snow",
            table_background_color="snow",
            table_font_size=10,
        )
        .tab_style(
            style=[style.fill(color="snow"), style.text(color="black")],
            locations=loc.body(
                columns=list(range(0, len(df_table.columns))),
                rows=list(range(0, len(df_table))),
            ),
        )
        .tab_style(
            style=[style.fill(color="lightgray"), style.text(color="black")],
            locations=loc.body(
                columns=[0],
                rows=list(range(0, len(df_table))),
            ),
        )
    )


def df_as_great_table2(
    df_table: pd.DataFrame,
    title=None,
    subtitle=None,
    rowname_col: str | None = None,
    groupname_col: str | None = None,
):
    """Used by monitoring report"""
    rowname_col = rowname_col or "label"
    groupname_col = groupname_col or "visit_code"
    return (
        GT(df_table, rowname_col=rowname_col, groupname_col=groupname_col)
        .tab_header(title=html(f'<div class="table-header">{title}</div>'), subtitle=subtitle)
        .cols_align(align="left", columns=[0, 1])
        .cols_align(align="right", columns=list(range(2, len(df_table.columns))))
        .opt_stylize(style=3)
        .opt_row_striping(row_striping=True)
        .opt_vertical_padding(scale=1.2)
        .opt_horizontal_padding(scale=1.0)
        .tab_options(
            table_width="100%",
            stub_background_color="snow",
            row_group_border_bottom_style="hidden",
            row_group_padding=0.5,
            row_group_background_color="snow",
            table_background_color="snow",
            table_font_size=10,
        )
        .tab_style(
            style=[style.fill(color="snow"), style.text(color="black")],
            locations=loc.body(
                columns=list(range(0, len(df_table.columns))),
                rows=list(range(0, len(df_table))),
            ),
        )
    )



from edc_listboard.views import ScreeningListboardView

from meta_edc.meta_version import get_meta_version


class ListboardView(ScreeningListboardView):
    listboard_model = "meta_screening.screeningpartone"
    navbar_selected_item = "screened_subject"

    def get_context_data(self, **kwargs) -> dict:
        kwargs.update(meta_version=get_meta_version())
        return super().get_context_data(**kwargs)

    def get_search_fields(self) -> tuple[str, ...]:
        fields = super().get_search_fields()
        return *fields, "hospital_identifier__exact"



# Generated by Django 3.0.6 on 2020-05-16 23:07

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0036_auto_20200517_0150"),
    ]

    operations = [
        migrations.RenameField(
            model_name="historicalmissedvisit",
            old_name="contact_attempts_made",
            new_name="contact_attempts_count",
        ),
        migrations.RenameField(
            model_name="missedvisit",
            old_name="contact_attempts_made",
            new_name="contact_attempts_count",
        ),
    ]



from clinicedc_constants import NO, NOT_APPLICABLE, OTHER
from django.db import models
from edc_constants.choices import YES_NO
from edc_glucose.model_mixin_factories import (
    fasting_model_mixin_factory,
    fbg_model_mixin_factory,
)
from edc_model.models import BaseUuidModel
from edc_utils import formatted_date

from meta_lists.constants import ACCUCHEK, HEMACUE
from meta_lists.models import DiagnosticDevices

from ..choices import ENDPOINT_CHOICES
from ..constants import AMENDMENT_DATE
from ..model_mixins import CrfModelMixin


class GlucoseFbg(
    CrfModelMixin,
    fasting_model_mixin_factory(
        None,
        fasting=models.CharField(
            verbose_name="Has the participant fasted?",
            max_length=15,
            choices=YES_NO,
            null=True,
            blank=False,
            help_text="As reported by patient",
        ),
    ),
    fbg_model_mixin_factory("fbg"),
    BaseUuidModel,
):
    fbg_performed = models.CharField(
        verbose_name="Was the FBG test performed?",
        max_length=15,
        choices=YES_NO,
    )

    fbg_not_performed_reason = models.CharField(
        verbose_name="If NO, provide reason",
        max_length=150,
        default="",
        blank=True,
    )

    fasting_duration_estimated = models.CharField(
        max_length=15,
        default=NO,
        editable=False,
        help_text=(
            "Set to YES for existing values before duration "
            "question was added to the form, otherwise NO"
        ),
    )

    endpoint_today = models.CharField(
        verbose_name="Has the participant reached a study endpoint today?",
        max_length=25,
        choices=ENDPOINT_CHOICES,
        default=NOT_APPLICABLE,
        help_text=(
            f"Response is applicable if reporting after {formatted_date(AMENDMENT_DATE)} "
            "and both the FBG and OGTT were performed"
        ),
    )

    repeat_fbg_date = models.DateField(
        null=True,
        blank=True,
        help_text="Date should be within 1 week of report date",
    )

    fbg_diagnostic_device = models.ForeignKey(
        DiagnosticDevices,
        on_delete=models.PROTECT,
        null=True,
        blank=False,
        limit_choices_to={"name__in": [ACCUCHEK, HEMACUE, OTHER, NOT_APPLICABLE]},
    )

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Glucose (FBG)"
        verbose_name_plural = "Glucose (FBG)"



from clinicedc_constants import NO, NOT_APPLICABLE, PENDING, TBD, YES
from django.contrib import admin
from django.db.models import Q
from edc_appointment.admin import AppointmentListFilter

from ..constants import PENDING_REPEAT


class EligibilityPending(admin.SimpleListFilter):
    title = "P3 pending"

    parameter_name = "eligibility_pending"

    def lookups(self, request, model_admin):  # noqa: ARG002
        return (
            (PENDING, "Pending"),
            (PENDING_REPEAT, "Pending repeat GLU"),
        )

    def queryset(self, request, queryset):  # noqa: ARG002
        if self.value() == PENDING_REPEAT:
            return queryset.filter(
                eligible_part_one=YES,
                eligible_part_two=YES,
                eligible_part_three=TBD,
                part_three_report_datetime__isnull=False,
                repeat_glucose_opinion=YES,
            )
        if self.value() == PENDING:
            return queryset.filter(
                eligible_part_one=YES,
                eligible_part_two=YES,
                eligible_part_three=TBD,
                agree_to_p3=YES,
                p3_ltfu=NOT_APPLICABLE,
                part_three_report_datetime__isnull=True,
            )
        return queryset


class P3LtfuListFilter(admin.SimpleListFilter):
    title = "Contact for P3?"

    parameter_name = "p3_ltfu_custom"

    def lookups(self, request, model_admin):  # noqa: ARG002
        return (
            (YES, "Yes"),
            (NO, "No, lost contact"),
        )

    def queryset(self, request, queryset):  # noqa: ARG002
        if self.value() == YES:
            return queryset.filter(
                eligible_part_one=YES,
                eligible_part_two=YES,
                eligible_part_three=TBD,
                p3_ltfu=NOT_APPLICABLE,
            )
        if self.value() == NO:
            return queryset.filter(
                eligible_part_one=YES,
                eligible_part_two=YES,
                eligible_part_three=TBD,
                p3_ltfu=YES,
            )
        if self.value() == NO:
            return queryset.exclude(
                Q(eligible_part_one=TBD)
                | Q(eligible_part_two=TBD)
                | Q(eligible_part_three=TBD)
            )
        return queryset


class P3ApptListFilter(AppointmentListFilter):
    title = "Part 3 appointment"
    parameter_name = "p3_appt_datetime"
    field_name = "appt_datetime"

    @property
    def extra_queryset_options(self):
        return dict(
            eligible_part_one=YES,
            eligible_part_two=YES,
            eligible_part_three=TBD,
        )



import sys

from django.core.management import BaseCommand

from meta_analytics.dataframes import GlucoseEndpointsByDate


class Command(BaseCommand):
    def handle(self, *args, **options):  # noqa: ARG002
        sys.stdout.write("Generating endpoints...\n")
        cls = GlucoseEndpointsByDate()
        cls.run()
        cls.to_model()
        sys.stdout.write("Generating endpoints... done.\n")
